Package: twilight
Version: 1.88.0
Title: Estimation of local false discovery rate
Authors@R: person(given="Stefanie",family="Senger",
  email="stefanie.scheid@gmx.de",role=c("cre","aut"),
  comment=c(ORCID="0000-0003-4144-1040"))
Description: In a typical microarray setting with gene expression data
        observed under two conditions, the local false discovery rate
        describes the probability that a gene is not differentially
        expressed between the two conditions given its corrresponding
        observed score or p-value level. The resulting curve of
        p-values versus local false discovery rate offers an insight
        into the twilight zone between clear differential and clear
        non-differential gene expression. Package 'twilight' contains
        two main functions: Function twilight.pval performs a
        two-condition test on differences in means for a given input
        matrix or expression set and computes permutation based
        p-values. Function twilight performs a stochastic downhill
        search to estimate local false discovery rates and effect size
        distributions. The package further provides means to filter for
        permutations that describe the null distribution correctly.
        Using filtered permutations, the influence of hidden
        confounders could be diminished.
Depends: R (>= 2.10)
Imports: Biobase, graphics, grDevices, splines, stats
Suggests: golubEsets (>= 1.4.2), vsn (>= 1.7.2)
License: GPL (>= 2)
URL: http://compdiag.molgen.mpg.de/software/twilight.shtml
biocViews: Microarray, DifferentialExpression, MultipleComparison
git_url: https://git.bioconductor.org/packages/twilight
git_branch: RELEASE_3_23
git_last_commit: 3b91675
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:17:52 UTC; biocbuild
Author: Stefanie Senger [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-4144-1040>)
Maintainer: Stefanie Senger <stefanie.scheid@gmx.de>
